package no.integration.svv.config;

import no.integration.svv.properties.AuthConfigProperties;
import no.integration.svv.properties.WebServiceConfigProperties;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.ws.config.annotation.EnableWs;
import org.springframework.ws.config.annotation.WsConfigurerAdapter;
import org.springframework.ws.transport.http.MessageDispatcherServlet;
import org.springframework.ws.wsdl.wsdl11.SimpleWsdl11Definition;
import org.springframework.ws.wsdl.wsdl11.Wsdl11Definition;

/*
Contains the endpoint webservice configuration and interceptor beans
*/

@EnableWs
@Configuration
public class WebServiceConfig extends WsConfigurerAdapter {

    private final WebServiceConfigProperties webServiceConfig;
    private final AuthConfigProperties  authConfigProperties;

    public WebServiceConfig(WebServiceConfigProperties webServiceConfig, AuthConfigProperties authConfigProperties, AuthConfigProperties authConfigProperties1) {
        this.webServiceConfig = webServiceConfig;
        this.authConfigProperties = authConfigProperties1;
    }

//    @Override
//    public void addInterceptors(List<EndpointInterceptor> interceptors) {
//        try {
//            interceptors.add(new AuthenticationInterceptor(authConfigProperties));
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

    @Bean
    public ServletRegistrationBean<MessageDispatcherServlet> messageDispatcherServlet(ApplicationContext applicationContext) {
        MessageDispatcherServlet servlet = new MessageDispatcherServlet();
        servlet.setApplicationContext(applicationContext);
        servlet.setTransformWsdlLocations(true);
        servlet.setTransformSchemaLocations(true);
        return new ServletRegistrationBean<>(servlet, webServiceConfig.getServlet());
    }

    @Bean(name = "OppdateringService")
    public Wsdl11Definition defaultWsdl11Definition() {
        SimpleWsdl11Definition wsdl11Definition = new SimpleWsdl11Definition();
        wsdl11Definition.setWsdl(new ClassPathResource(webServiceConfig.getWsdlLocation()));


        return wsdl11Definition;
    }

}
