package no.integration.svv.utils.validator;

import no.integration.svv.exception.ValidationException;
import no.vegvesen.saksarkiv.felles.typer._1.JournalpostKvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.KvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.MappeKvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.NyttVedleggKvitteringType;

public class ResponseValidator {

    public static final String NY_JOURNALPOST_ENDPOINT = "NyJournalpost";
    public static final String NY_SAKSMAPPE_ENDPOINT = "NySaksmappe";
    public static final String NY_DOKUMENT_ENDPOINT = "NyDokument";
    public static final String NY_JPSTATUS_ENDPOINT = "EndreStatus";

    public static void validateJournalPostResponse(JournalpostKvitteringType response) throws ValidationException {

        if (response == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "body", "Missing value");

        }
        if (response.getRegistrering() == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getRegistrering", "Missing value");

        }
        if (response.getRegistrering().getDokumentId() == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getDokumentId", "Missing value");
        }
    }

    public static void validateSaksmappeResponse(MappeKvitteringType response) throws ValidationException {

        if (response == null) {
            throw new ValidationException(NY_SAKSMAPPE_ENDPOINT, "body", "Missing value");

        }
        if (response.getMappeId() == null) {
            throw new ValidationException(NY_SAKSMAPPE_ENDPOINT, "getMappeId", "Missing value");

        }
        if (response.getMappeId().getSaksnummer() == null) {
            throw new ValidationException(NY_SAKSMAPPE_ENDPOINT, "getSaksnummer", "Missing value");
        }
    }

    public static void validateDokumentResponse(NyttVedleggKvitteringType response) throws ValidationException {

        if (response == null) {
            throw new ValidationException(NY_DOKUMENT_ENDPOINT, "body", "missing value");
        }
        if (response.getDokument() == null || response.getDokument().size() == 0) {
            throw new ValidationException(NY_DOKUMENT_ENDPOINT, "getDokument", "missing value");
        }

    }

    public static void validateJPStatusResponse(KvitteringType response) throws ValidationException {

        if (response.toString() == " ") {
            throw new ValidationException(NY_JPSTATUS_ENDPOINT, "body", "Invalid response");
        }

    }
}
