package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Journalpost;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpost;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpostResponse;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.felles.typer._1.FagsystemType;
import no.vegvesen.saksarkiv.felles.typer._1.JournalpostKvitteringType;
import no.vegvesen.saksarkiv.journalpost.melding.opprett._1.OpprettJournalpostRequestType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import javax.xml.datatype.DatatypeConfigurationException;
import java.text.ParseException;

import static org.junit.jupiter.api.Assertions.assertNull;

class JournalPostTest {

    @Test
    void nyJournalPostRequestToMime() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();

        OpprettJournalpostRequestType mimeOutput = JournalPost.endpointToMimeOppJPT(geoInput);
        Assertions.assertNotNull(mimeOutput);
        Assertions.assertNotNull(mimeOutput.getJournalpost());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender().get(0).getMetadata());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender().get(0).getMetadata().getEnhetsregisteridentifikator());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender().get(0).getMetadata().getFolkeregisteridentifikator());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender().get(0).getMetadata().getKorrespondanseparttype());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender().get(0).getMetadata().getPostadresse());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getAvsender().get(0).getIdentifikator());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getMottaker());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getMottaker().get(0).getMetadata());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getMottaker().get(0).getMetadata().getKorrespondanseparttype());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getMottaker().get(0).getMetadata().getPostadresse());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getMottaker().get(0).getIdentifikator());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getJournalpostMetadata().getJournalpostType());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getMappe());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getMappe().getMappeId());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getRegistreringstypespesifikkeAtributter());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getRegistreringstypespesifikkeAtributter().getAttributt());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getRegistreringstypespesifikkeAtributter().getAttributt().get(0));
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming().getKode());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming().getKode().getHjemmel());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming().getHandling());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getAnsvar().getAnsvarligEnhet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getAnsvar().getAnsvarligPerson());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getDokumentstatus());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getDokumenttypespesifikkeAttributter());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getDokumenttypespesifikkeAttributter().getAttributt());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getDokumenttypespesifikkeAttributter().getAttributt().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getEier());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getForfattere());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getForfattere().getForfatter());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getVegobjekter());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getVegobjekter().getVegobjekt());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getMetadata().getVegobjekter().getVegobjekt().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getFil());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getFil().getFiltype());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getDokument().getFil().getInnhold());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getBruker());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getBruker().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getBruker().get(0).getRettighet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getGruppe());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getGruppe().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getGruppe().get(0).getRettighet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getOrgenhet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getOrgenhet().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getOrgenhet().get(0).getRettighet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getTilgangsgruppe());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getRolle().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getRolle().get(0).getRettighet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getVegobjekter());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getVegobjekter().getVegobjekt());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getVegobjekter().getVegobjekt().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getForfattere());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getForfattere().getForfatter());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getForfattere().getForfatter().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getEier());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getDokumenttypespesifikkeAttributter());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getDokumenttypespesifikkeAttributter().getAttributt());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getDokumenttypespesifikkeAttributter().getAttributt().get(0));
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getMetadata().getDokumentstatus());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getFil());
//        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg().get(0).getFil().getFiltype());


        Assertions.assertEquals(geoInput.getJournalpost().getTittel(), mimeOutput.getJournalpost().getBasisMetadata().getTittel());
        Assertions.assertEquals(geoInput.getJournalpost().getOffentligTittel(), mimeOutput.getJournalpost().getBasisMetadata().getOffentligTittel());
        Assertions.assertEquals(geoInput.getJournalpost().getTilgangsgruppeNavn(), mimeOutput.getJournalpost().getTilgang().getTillatelse().getTilgangsgruppe().getGruppenavn());
        Assertions.assertEquals(geoInput.getJournalpost().getDokumentetsDato(), mimeOutput.getJournalpost().getJournalpostMetadata().getDokumentdato());
        Assertions.assertEquals(geoInput.getJournalpost().getForfallsdato(), mimeOutput.getJournalpost().getJournalpostMetadata().getForfalldato());
    }


    @Test
    @DisplayName("Throw validation exception when Body is null")
    void nyGeoInputIsNull() throws DatatypeConfigurationException, ParseException {

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(null);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("Body", thrown.getFieldName());

    }

    @Test
    @DisplayName("Throw validation exception when JournalPost is null")
    void nyJournalPostIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        geoInput.setJournalpost(null);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getJournalpost", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when JournalPost.getTilleggsinformasjon is null")
    void tilleggsinformasjonIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        journalpost.setTilleggsinformasjon(null);
        geoInput.setJournalpost(journalpost);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    JournalPost.endpointToMimeOppJPT(geoInput);
                }
        );

        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getTilleggsinformasjon", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when JournalPost.getJournalposttype is null")
    void journalposttypeIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        journalpost.setJournalposttype(null);
        geoInput.setJournalpost(journalpost);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    JournalPost.endpointToMimeOppJPT(geoInput);
                }
        );

        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getJournalposttype", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when JournalPost.getSkjerming is null")
    void skjermingIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        journalpost.setSkjerming(null);
        geoInput.setJournalpost(journalpost);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    JournalPost.endpointToMimeOppJPT(geoInput);
                }
        );
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getSkjerming", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when JournalPost.getReferanseSakSystemID is null")
    void referanseSakSystemIDIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        journalpost.setReferanseSakSystemID(null);
        geoInput.setJournalpost(journalpost);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getReferanseSakSystemID", thrown.getFieldName());
    }


    @Test
    @DisplayName("Throw validation exception when JournalPost.getKorrespondansepart is null")
    void korrespondansepartlIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        journalpost.setKorrespondansepart(null);
        geoInput.setJournalpost(journalpost);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getKorrespondansepart", thrown.getFieldName());
    }

    @Test
    void nyJournalpostResponseToSuFa() throws DatatypeConfigurationException, ParseException {

        /*Set up* */
        JournalpostKvitteringType mimeInput  = JournalPostTestFixture.jpResponse();
        NyJournalpostResponse  geoOutput = JournalPost.nyJournalpostResponseToSuFa(mimeInput);

        Assertions.assertNotNull(geoOutput);
        Assertions.assertNotNull(geoOutput.getReturn());
//        Assertions.assertNotNull(geoOutput.getReturn().getSaksnr());
        Assertions.assertNotNull(geoOutput.getReturn().getJournalpostnummer());

//        Assert
        Assertions.assertEquals(geoOutput.getReturn().getSystemID(), mimeInput.getRegistrering().getDokumentId());
        Assertions.assertEquals(geoOutput.getReturn().getJournalpostnummer(), mimeInput.getRegistrering().getJournalpostId().getLopenummer());

        String saksNumRes = mimeInput.getRegistrering().getJournalpostId().getMappeId().getSaksnummer();

//        Assertions.assertEquals(geoOutput.getReturn().getSaksnr().getSaksaar(), new BigInteger(saksNumRes.substring(0, 2)));
//        Assertions.assertEquals(geoOutput.getReturn().getSaksnr().getSakssekvensnummer(),new BigInteger(saksNumRes.substring(3,saksNumRes.length())));

    }


    void nyJournalpostResponseToSuFa_1() {

        /*Set up
         * */

        JournalpostKvitteringType input = new JournalpostKvitteringType();
//        JournalpostIdentifikatorType journalpostIdentifikatorType = new JournalpostIdentifikatorType();
//        journalpostIdentifikatorType.setDokumentId("XXXXXXX");
//        input.setRegistrering(journalpostIdentifikatorType);

//        Run
        NyJournalpostResponse output = JournalPost.nyJournalpostResponseToSuFa(input);


//        Assert

        //Null check(won't fail if input is Null)
        assertNull(output.getReturn().getSystemID());
        //assertEquals(input.getRegistrering().getDokumentId(),output.getReturn().getSystemID());
        //assertEquals(input.getRegistrering().getDokumentId(),output.getReturn().getSystemID());

    }


    @Test
    void getFagSystemMapping() throws DatatypeConfigurationException, ParseException {
        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        FagsystemType fagSystemMapping = JournalPost.getFagSystemMapping(geoInput);

        Assertions.assertNotNull(fagSystemMapping);
        Assertions.assertNotNull(fagSystemMapping.getFagsystemSluttbruker());



    }


}

