package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Saksmappe;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappe;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappeResponse;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.integration.svv.utils.validator.ResponseValidator;
import no.vegvesen.saksarkiv.felles.typer._1.MappeKvitteringType;
import no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.OpprettSaksmappeRequestType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import javax.xml.datatype.DatatypeConfigurationException;
import java.math.BigInteger;
import java.text.ParseException;

class SaksMappeTest {

    @Test
    void endpointToMimeOppSM() throws DatatypeConfigurationException, ParseException {
        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();

        OpprettSaksmappeRequestType mimeOutput = SaksMappe.endpointToMimeOppSM(geoInput);

        Assertions.assertNotNull(mimeOutput);
        Assertions.assertNotNull(mimeOutput.getFagsystem());
        Assertions.assertNotNull(mimeOutput.getFagsystem().getFagsystemNavn());
        Assertions.assertNotNull(mimeOutput.getFagsystem().getFagsystemSluttbruker());
        Assertions.assertNotNull(mimeOutput.getFagsystem().getFagsystemSluttbruker().getBrukerident());
        Assertions.assertNotNull(mimeOutput.getFagsystem().getFagsystemSluttbruker().getNavn());

        Assertions.assertNotNull(mimeOutput.getSaksmappe());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getTilgang());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getTilgang().getTillatelse());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getTilgang().getTillatelse().getTilgangsgruppe());

        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getSkjerming());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().getHjemmel());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getMappetypespesifikkeAttributter().getAttributt());


        Assertions.assertEquals(geoInput.getMappe().getOffentligTittel(), mimeOutput.getSaksmappe().getMetadata().getTittel());
        Assertions.assertEquals(geoInput.getMappe().getTittel(), mimeOutput.getSaksmappe().getMetadata().getOffentligTittel());
        Assertions.assertEquals(geoInput.getMappe().getMappetype().getKodeverdi(), mimeOutput.getSaksmappe().getMetadata().getMappeType());
        Assertions.assertEquals(geoInput.getMappe().getAdministrativEnhetInit(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligEnhet().getAnsvarskode());
        Assertions.assertEquals(geoInput.getMappe().getAdministrativEnhet(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligEnhet().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getSaksansvarlig(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligPerson().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getSaksansvarligInit(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligPerson().getBrukerident());


//        geoInput.getMappe().getTilleggsinformasjon().getListe().forEach(item -> {
//            if (item != null) {
//                Assertions.assertEquals(item.getTilgangsgruppeNavn(), mimeOutput.getSaksmappe().getTilgang().getTillatelse().getTilgangsgruppe().getGruppenavn());
////            Assertions.assertEquals(item.getInformasjonstype().getKodeverdi(), mimeOutput.getSaksmappe().getMetadata());
//                Assertions.assertEquals(item.getInformasjonstype().getKodebeskrivelse(), mimeOutput.getSaksmappe().getMetadata().getBeskrivelse());
//            }
//        });

        Assertions.assertEquals(geoInput.getMappe().getSkjerming().getTilgangsrestriksjon().getKodeverdi(), mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getSkjerming().getTilgangsrestriksjon().isErGyldig(), mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().isKorrespondanseparterSkjermet());
        Assertions.assertEquals(geoInput.getMappe().getSkjerming().getSkjermingshjemmel(), mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().getHjemmel().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getTilgangsgruppeNavn(), mimeOutput.getSaksmappe().getTilgang().getTillatelse().getTilgangsgruppe().getGruppenavn());


    }

    @Test
    @DisplayName("Throw validation exception when Body is null")
    void nyGeoInputIsNull() throws DatatypeConfigurationException, ParseException {

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            SaksMappe.endpointToMimeOppSM(null);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("Body", thrown.getFieldName());

    }

    @Test
    @DisplayName("Throw validation exception when SaksMappe is null")
    void nySaksmappeIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        geoInput.setMappe(null);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            SaksMappe.endpointToMimeOppSM(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getMappetype is null")
    void mappeTypeIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setMappetype(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );

        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getMappetype", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getTilleggsinformasjon is null")
    void tilleggsinformasjonIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setTilleggsinformasjon(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );

        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getTilleggsinformasjon", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getSaksnr is null")
    void saksnrIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setSaksnr(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getSaksnr", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getSkjerming is null")
    void skjermingIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setSkjerming(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getSkjerming", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getReferanseEksternNoekkel is null")
    void referanseEksternNoekkelIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setReferanseEksternNoekkel(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getReferanseEksternNoekkel", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getSaksstatus is null")
    void saksstatuslIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setSaksstatus(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getSaksstatus", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getMappe.getKlasse is null")
    void klasselIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        Saksmappe saksmappe = new Saksmappe();
        saksmappe.setKlasse(null);

        geoInput.setMappe(saksmappe);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                    SaksMappe.endpointToMimeOppSM(geoInput);
                }
        );
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe.getKlasse", thrown.getFieldName());
    }


    @Test
    void nySaksmappeResponseToSuFa() {

        MappeKvitteringType mimeInput = new MappeKvitteringType();

        Assertions.assertNotNull(mimeInput);
        Assertions.assertNotNull(mimeInput.getMappeId());
        Assertions.assertNotNull(mimeInput.getMappeId().getSaksnummer());


        NySaksmappeResponse output = new NySaksmappeResponse();
        String saksNumRes = mimeInput.getMappeId().getSaksnummer();
        Assertions.assertEquals(output.getReturn().getSaksnr().getSaksaar(), (new BigInteger(saksNumRes.substring(0, 2))));
        Assertions.assertEquals(output.getReturn().getSaksnr().getSakssekvensnummer(), (new BigInteger(saksNumRes.substring(3, saksNumRes.length()))));

    }

    @Test
    @DisplayName("Throw validation exception when Body is null")
    void mimeInputIsNull() throws DatatypeConfigurationException, ParseException {

        MappeKvitteringType mimeInput = new MappeKvitteringType();

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            SaksMappe.nySaksmappeResponseToSuFa(null);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(ResponseValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("Body", thrown.getFieldName());

    }

    @Test
    @DisplayName("Throw validation exception when getMappeId is null")
    void mappeIsNull() throws DatatypeConfigurationException, ParseException {


        MappeKvitteringType mimeInput = new MappeKvitteringType();
        mimeInput.setMappeId(null);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            SaksMappe.nySaksmappeResponseToSuFa(mimeInput);
        });

        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(ResponseValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappeId", thrown.getFieldName());

    }

    @Test
    public void nySaksmappeResponseTest() throws DatatypeConfigurationException, ParseException {

        MappeKvitteringType mimeInput = SaksmappeTestFixture.saksmappeResponse();
        NySaksmappeResponse geoOutput = SaksMappe.nySaksmappeResponseToSuFa(mimeInput);

        Assertions.assertNotNull(geoOutput);
        Assertions.assertNotNull(geoOutput.getReturn());
        Assertions.assertNotNull(geoOutput.getReturn().getSaksnr());

        String saksNumRes = mimeInput.getMappeId().getSaksnummer();
        Assertions.assertEquals(new BigInteger(saksNumRes.substring(0, 2)), geoOutput.getReturn().getSaksnr().getSaksaar());
        Assertions.assertEquals(new BigInteger(saksNumRes.substring(3,saksNumRes.length())), geoOutput.getReturn().getSaksnr().getSakssekvensnummer());

    }
}